@@ -72,6 +72,7 @@ class Specter:
             # unique id that will be used in wallets path in Bitcoin Core
             # empty by default for backward-compatibility
             "uid": "",
+            "unit": "btc",
         }
 
         # creating folders if they don't exist
@@ -119,6 +120,7 @@ class Specter:
                             ("getmempoolinfo", None),
                             ("uptime", None),
                             ("getblockhash", 0),
+                            ("scantxoutset", "status", []),
                         ]
                     )
                 ]
@@ -131,6 +133,9 @@ class Specter:
                     self._info['blockfilterindex'] = True
                 except:
                     self._info['blockfilterindex'] = False
+                self._info["utxorescan"] = (res[5]["progress"]
+                                            if res[5] is not None and "progress" in res[5]
+                                            else None)
                 self._is_running = True
             except Exception as e:
                 self._info = {"chain": None}
@@ -176,6 +181,12 @@ class Specter:
                     chain=chain
                 )
 
+    def abortrescanutxo(self):
+        self.cli.scantxoutset("abort", [])
+        # Bitcoin Core doesn't catch up right away
+        # so app.specter.check() doesn't work
+        self._info["utxorescan"] = None
+
     def clear_user_session(self):
         self.device_manager = None
         self.wallet_manager = None
@@ -268,12 +279,30 @@ class Specter:
         if url and not url.endswith("/"):
             # make sure the urls end with a "/"
             url += "/"
+        # a few dummy checks:
+        # no schema and not local
+        if "://" not in url and not url.startswith("/"):
+            url = "http://"+url
+        # wrong ending:
+        if url.endswith("/hwi/settings/"):
+            url = url.replace("/hwi/settings/", "/hwi/api/")
+        # no ending
+        if not url.endswith("/hwi/api/"):
+            url += "hwi/api/"
+
         if user.id == 'admin':
             self.config["hwi_bridge_url"] = url
             self._save()
         else:
             user.set_hwi_bridge_url(self, url)
 
+    def update_unit(self, unit, user):
+        if user.id == 'admin':
+            self.config["unit"] = unit
+            self._save()
+        else:
+            user.set_unit(self, unit)
+
     def add_new_user_otp(self, otp_dict):
         ''' adds an OTP for user registration '''
         if 'new_user_otps' not in self.config:
@@ -368,6 +397,20 @@ class Specter:
             else:
                 return ""
 
+    @property
+    def unit(self):
+        # TODO: Unify for user and admin
+        if (not current_user or current_user.is_anonymous) or current_user.is_admin:
+            if "unit" in self.config:
+                return self.config["unit"]
+            else:
+                return "btc"
+        else:
+            if "unit" in current_user.config:
+                return current_user.config["unit"]
+            else:
+                return "btc"
+
     def specter_backup_file(self):
         memory_file = BytesIO()
         with zipfile.ZipFile(memory_file, 'w') as zf:
